
from .execute import execute

from .template import Template
from .params import Params
from .snapshot import snapshot
from .embed_html import EmbedHTMLExporter as html_embed_img

from .move import move




__name__ = 'notebook_snapshot'
name_url = __name__.replace('_', '-')

__packages__ = [__name__]
__version__ = '0.1.2'
__description__ = 'Execute a notebook, snapshot it as html (option: embedding all images) and store the result'
__author__ = 'oscar6echo'
__author_email__ = 'olivier.borderies@gmail.com'
__url__ = 'https://github.com/{}/{}'.format(__author__,
                                            name_url)
__download_url__ = 'https://github.com/{}/{}/tarball/{}'.format(__author__,
                                                                name_url,
                                                                __version__)
__keywords__ = ['jupyter', 'nbconvert', 'embed images', 'execute']
__license__ = 'MIT'
__classifiers__ = ['Development Status :: 4 - Beta',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'
                   ]
__include_package_data__ = True
__package_data__ = {
    'templates':
    ['templates/hide_prompt.css',
     'templates/main.tpl.html',
     'templates/getCellsFromString.js',
     'templates/notebook.css',
     'templates/ribbon.css'
     ]
}




import sys
import os
import re
import jinja2 as jj

from .params import Params


class Template:
    """
    """

    def __init__(self,
                 template_str=None,
                 params=None
                 ):
        """
        """

        dir_file = os.path.dirname(os.path.abspath(__file__))
        dir_template = os.path.join(dir_file, 'templates')

        loader = jj.FileSystemLoader(dir_template)

        env = jj.Environment(loader=loader,
                             variable_start_string='__$',
                             variable_end_string='$__',
                             block_start_string='{-%',
                             block_end_string='%-}'
                             )

        if not template_str:
            template_name = 'main.tpl.html'
            template = env.get_template(template_name)
        else:
            template = env.from_string(template_str)

        if isinstance(params, Params):
            data = params.to_dict()
        else:
            msg = 'If params is not an instance of Params it must be a dict valid for the template used'
            assert isinstance(params, dict), msg
            data = params

        self.content = template.render(data=data)

    def save(self,
             save_folder=None,
             save_name=None,
             verbose=False):
        """
        """
        if not save_folder:
            save_folder = 'dump'

        if not os.path.exists(save_folder):
            os.makedirs(save_folder)

        if not save_name:
            save_name = 'my_template.tpl'

        file_path = os.path.join(save_folder, save_name)
        with open(file_path, 'w') as f:
            f.write(self.content)

        if verbose:
            msg = 'template {} created'
            msg = msg.format(file_path)

        self.path = file_path




import os
import nbformat
import warnings

import datetime as dt

from IPython.display import display, Markdown
from nbconvert.preprocessors import ExecutePreprocessor
from nbconvert.preprocessors import CellExecutionError


def execute(path_nb,
            dated=True,
            timeout=600,
            kernel_name='python3',
            verbose=False):
    """
    """

    folder = os.path.dirname(path_nb)
    filename = os.path.basename(path_nb)

    now = dt.datetime.now().strftime('%Y%m%d_%H%M%S.%f')[:-3]
    if dated:
        save_name = '{}_{}'.format(now, filename)
    else:
        save_name = filename

    # path_output
    path_out = os.path.join(folder, save_name)

    # load nb
    with open(path_nb) as f:
        nb = nbformat.read(f, as_version=4)

    # create ep
    ep = ExecutePreprocessor(timeout=timeout,
                             kernel_name=kernel_name)

    # run catching errors
    try:
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            out = ep.preprocess(nb, {'metadata': {'path': '.'}})
    except CellExecutionError:
        out = None
        msg = 'Error executing notebook {}\nSee notebook {} for traceback.'
        msg = msg.format(path_nb, path_out)
        print(msg)
        raise
    finally:
        with open(path_out, mode='w') as f:
            nbformat.write(nb, f)

    if verbose:
        md = 'Notebook **{}** run and saved as **{}**'
        md = md.format(path_nb, path_out)
        display(Markdown(md))

    return path_out, now




import os
import glob
import shutil


def move(tag,
         folder='store',
         verbose=False):
    """
    """
    if not os.path.exists(folder):
        os.makedirs(folder)

    filenames = glob.glob('{}*.ipynb'.format(tag)) + \
                glob.glob('{}*.html'.format(tag))
    for f in filenames:
        shutil.move(f, folder)

    if verbose:
        msg = 'Moved the files below to folder {}'
        msg =msg.format(folder)
        print(msg)
        print('\n'.join(filenames))




import sys
import subprocess as sp


def snapshot(path_nb,
             path_template=None,
             embed_img=True,
             ):
    """
    """
    if embed_img:
        exporter = 'notebook_snapshot.html_embed_img'
    else:
        exporter = 'html'

    cmd = 'jupyter nbconvert --to {} --template {} {}'
    cmd = cmd.format(exporter, path_template, path_nb)
    print(cmd)
    args = cmd.split()
    # os.system(cmd)

    with sp.Popen(args, stdout=sp.PIPE, stderr=sp.PIPE) as process:
        # Read stdout character by character, as it includes real-time progress updates
        for c in iter(lambda: process.stdout.read(1), b''):
            sys.stdout.write(c.decode(sys.stdout.encoding))
        # Read stderr line by line, because real-time does not matter
        for line in iter(process.stderr.readline, b''):
            sys.stderr.write(line.decode(sys.stderr.encoding))




import json

from copy import deepcopy as copy


class Params:
    """
    """

    def __init__(self,
                 verbose=False,
                 **kwargs):
        """
        """
        # attributes
        self.is_ribbon = True
        self.ribbon_color = 'blue'
        self.ribbon_text = 'Draft'
        self.ribbon_position = 'top-right'

        self.hide_code = False

        self.important_border = '3px'
        self.important_color = 'red'
        self.important_background_color = '#f7f7f7'

        for k, v in kwargs.items():
            setattr(self, k, v)
        self.valid = self.check(verbose=verbose)

    def check(self, verbose=False):
        """
        """
        isRibbon = isinstance(self.is_ribbon, bool)
        li_ribbon_color = ['white', 'black', 'grey', 'blue',
                           'green', 'turquoise', 'purple', 'red', 'orange', 'yellow']
        isRibbonColor = self.ribbon_color in li_ribbon_color
        isRibbonText = isinstance(self.ribbon_text, str)
        li_ribbon_pos = ['top-right', 'top-left', 'bottom-right', 'bottom-left']
        isRibbonPosition = self.ribbon_position in li_ribbon_pos

        isHideCode = isinstance(self.hide_code, bool)

        isImportantBorder = isinstance(self.important_border, str)
        isImportantColor = isinstance(self.important_color, str)
        isImportantBackgroundColor = isinstance(
            self.important_background_color, str)

        isParams = all([isRibbon, isRibbonColor, isRibbonText, isRibbonPosition, isHideCode,
                        isImportantBorder, isImportantColor, isImportantBackgroundColor])

        if verbose:
            print('Params: isRibbon=', isRibbon)
            print('Params: isRibbonColor=', isRibbonColor)
            print('Params: isRibbonText=', isRibbonText)
            print('Params: isRibbonPosition=', isRibbonPosition)
            print('Params: isHideCode=', isHideCode)
            print('Params: isImportantBorder=', isImportantBorder)
            print('Params: isImportantColor=', isImportantColor)
            print('Params: isImportantBackgroundColor=', isImportantBackgroundColor)

            print('Params: isParams=', isParams)
        msg = 'Params must contain attributes to fill jinja2 template main.tpl.html'
        assert isParams, msg

        return True

    def to_dict(self):
        """
        """
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        return d

    def pprint(self, indent=2):
        """
        """
        d = json.dumps(self.to_dict(),
                       sort_keys=True,
                       indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())